@@ -17,13 +17,33 @@
 #include <Python.h>
 
 #include <errno.h>
+#include <limits.h>
 
 #include <loc/libloc.h>
 #include <loc/network.h>
+#include <loc/network-list.h>
 
 #include "locationmodule.h"
 #include "network.h"
 
+static PyObject* PyList_FromNetworkList(struct loc_network_list* networks) {
+	PyObject* list = PyList_New(0);
+	if (!networks)
+		return list;
+
+	while (!loc_network_list_empty(networks)) {
+		struct loc_network* network = loc_network_list_pop(networks);
+
+		PyObject* n = new_network(&NetworkType, network);
+		PyList_Append(list, n);
+
+		loc_network_unref(network);
+		Py_DECREF(n);
+	}
+
+	return list;
+}
+
 PyObject* new_network(PyTypeObject* type, struct loc_network* network) {
 	NetworkObject* self = (NetworkObject*)type->tp_alloc(type, 0);
 	if (self) {
@@ -114,11 +134,19 @@ static int Network_set_asn(NetworkObject* self, PyObject* value) {
 	long int asn = PyLong_AsLong(value);
 
 	// Check if the ASN is within the valid range
-	if (asn <= 0 || asn > UINT32_MAX) {
+	if (asn <= 0) {
 		PyErr_Format(PyExc_ValueError, "Invalid ASN %ld", asn);
 		return -1;
 	}
 
+#if (__WORDSIZE > 32)
+	// Check whether the input was longer than 32 bit
+	if (asn > UINT32_MAX) {
+		PyErr_Format(PyExc_ValueError, "Invalid ASN %ld", asn);
+		return -1;
+	}
+#endif
+
 	int r = loc_network_set_asn(self->network, asn);
 	if (r)
 		return -1;
@@ -154,13 +182,28 @@ static PyObject* Network_set_flag(NetworkObject* self, PyObject* args) {
 	Py_RETURN_NONE;
 }
 
+static PyObject* Network_exclude(NetworkObject* self, PyObject* args) {
+	NetworkObject* other = NULL;
+
+	if (!PyArg_ParseTuple(args, "O!", &NetworkType, &other))
+		return NULL;
+
+	struct loc_network_list* list = loc_network_exclude(self->network, other->network);
+
+	// Convert to Python objects
+	PyObject* obj = PyList_FromNetworkList(list);
+	loc_network_list_unref(list);
+
+	return obj;
+}
+
 static PyObject* Network_is_subnet_of(NetworkObject* self, PyObject* args) {
 	NetworkObject* other = NULL;
 
 	if (!PyArg_ParseTuple(args, "O!", &NetworkType, &other))
 		return NULL;
 
-	if (loc_network_is_subnet_of(self->network, other->network))
+	if (loc_network_is_subnet(other->network, self->network))
 		Py_RETURN_TRUE;
 
 	Py_RETURN_FALSE;
@@ -181,6 +224,26 @@ static PyObject* Network_get_first_address(NetworkObject* self) {
 	return obj;
 }
 
+static PyObject* PyBytes_FromAddress(const struct in6_addr* address6) {
+	struct in_addr address4;
+
+	// Convert IPv4 addresses to struct in_addr
+	if (IN6_IS_ADDR_V4MAPPED(address6)) {
+		address4.s_addr = address6->s6_addr32[3];
+
+		return PyBytes_FromStringAndSize((const char*)&address4, sizeof(address4));
+	}
+
+	// Return IPv6 addresses as they are
+	return PyBytes_FromStringAndSize((const char*)address6, sizeof(*address6));
+}
+
+static PyObject* Network_get__first_address(NetworkObject* self) {
+	const struct in6_addr* address = loc_network_get_first_address(self->network);
+
+	return PyBytes_FromAddress(address);
+}
+
 static PyObject* Network_get_last_address(NetworkObject* self) {
 	char* address = loc_network_format_last_address(self->network);
 
@@ -190,7 +253,19 @@ static PyObject* Network_get_last_address(NetworkObject* self) {
 	return obj;
 }
 
+static PyObject* Network_get__last_address(NetworkObject* self) {
+	const struct in6_addr* address = loc_network_get_last_address(self->network);
+
+	return PyBytes_FromAddress(address);
+}
+
 static struct PyMethodDef Network_methods[] = {
+	{
+		"exclude",
+		(PyCFunction)Network_exclude,
+		METH_VARARGS,
+		NULL,
+	},
 	{
 		"has_flag",
 		(PyCFunction)Network_has_flag,
@@ -241,6 +316,13 @@ static struct PyGetSetDef Network_getsetters[] = {
 		NULL,
 		NULL,
 	},
+	{
+		"_first_address",
+		(getter)Network_get__first_address,
+		NULL,
+		NULL,
+		NULL,
+	},
 	{
 		"last_address",
 		(getter)Network_get_last_address,
@@ -248,6 +330,13 @@ static struct PyGetSetDef Network_getsetters[] = {
 		NULL,
 		NULL,
 	},
+	{
+		"_last_address",
+		(getter)Network_get__last_address,
+		NULL,
+		NULL,
+		NULL,
+	},
 	{ NULL },
 };
 
