@@ -22,6 +22,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <unistd.h>
+#include <syslog.h>
 
 #include <loc/libloc.h>
 
@@ -32,6 +33,9 @@ int main(int argc, char** argv) {
 	if (err < 0)
 		exit(EXIT_FAILURE);
 
+	// Enable debug logging
+	loc_set_log_priority(ctx, LOG_DEBUG);
+
 	printf("version %s\n", VERSION);
 
 	loc_unref(ctx);
